@@ -3,7 +3,9 @@ use strict;
 use warnings FATAL => 'all';
 
 # Version :
-our $VERSION = '0.1002';
+our $VERSION = '0.1003';
+our $appVer = '0.1';
+our $SubVer = '03';
 
 # boidev.pl version :
 our $boidev_ver = '0.1001';
