@@ -14,10 +14,10 @@ use BlankOnDev::Utils::file;
 use BlankOnDev::enkripsi;
 use BlankOnDev::DataDev;
 use BlankOnDev::config::save;
+use BlankOnDev::Migration::bazaar2GitHub::tmp_cfg;
 
 # Version :
-require BlankOnDev::Version;
-our $VERSION = $BlankOnDev::Version::VERSION;
+our $VERSION = '0.1003';
 
 # Our vars :
 our $rilis = '';
@@ -33,6 +33,7 @@ our $r_bzrcfg = 0;
 our $r_gitcfg = 0;
 our $r_gpgcfg = 0;
 our $r_config = {};
+our $time_zone = 'Asia/Makassar';
 our $_setup = {};
 
 # Subroutine for option "prepare" :
@@ -53,6 +54,7 @@ sub _prepare {
     my $pkg = $r_config->{'pkg'};
     my $data = {
         'r_config' => {
+            'timezone' => $time_zone,
             'prepare' => $prepareCfg,
             'build' => {
                 'rilis' => $rilisCfg,
@@ -80,20 +82,21 @@ sub _gpg_genkey {
     # Get All Data Config :
     my $prepare = $allconfig->{'prepare'};
     my $build = $allconfig->{'build'};
-    my $build_rilis = $build-{'rilis'};
+    my $build_rilis = $build->{'rilis'};
     my $build_pgp = $build->{'pgp'};
     my $bzr = $allconfig->{'bzr'};
     my $git = $allconfig->{'git'};
     my $pkg = $allconfig->{'pkg'};
 
     # Enter Rilis :
-    boi_rilis();
+#    boi_rilis();
 
     # Check File Config :
-    my $file_cfg = $prefix_flcfg.$rilis.$prefix_flcfg;
+    my $file_cfg = $prefix_flcfg.$rilisCfg.$file_cfg_ext;
     my $loc_file = $dir_dev.$file_cfg;
     $filename_cfg = $file_cfg;
     $dirdev_cfg = $dir_dev;
+    print "Filename : $file_cfg\n";
     if (-e $loc_file) {
         # GPG Generate Key :
         my $gpg_cfg = gpg_config();
@@ -101,10 +104,11 @@ sub _gpg_genkey {
         # Define hash for new data :
         my $data = {
             'r_config' => {
+                'timezone' => $time_zone,
                 'prepare' => $prepare,
                 'build'   => {
                     'rilis' => $build_rilis,
-                    'pgp'   => $gpg_cfg,
+                    'gpg'   => $gpg_cfg->{'gpg'},
                 },
                 'bzr' => $bzr,
                 'git' => $git,
@@ -117,17 +121,149 @@ sub _gpg_genkey {
         BlankOnDev::config::save->gpg_genkey($data);
     }
 }
+# Subroutine for get list gpg auth :
+# ------------------------------------------------------------------------
+sub _gpg_auth {
+    # Get data Setup :
+    my $data_setup = data_setup();
+    my $dir_dev = $data_setup->{'dir_dev'};
+    my $prefix_flcfg = $data_setup->{'prefix_flcfg'};
+    my $file_cfg_ext = $data_setup->{'fileCfg_ext'};
+
+    # Get All Data Config :
+    my $build = $allconfig->{'build'};
+    my $build_rilis = $build->{'rilis'};
+    my $build_gpg = $build->{'gpg'};
+    my $name_gpg = $build_gpg->{'name'};
+    my $email_gpg = $build_gpg->{'email'};
+    my $passphrase_gpg = $build_gpg->{'passphrase'};
+
+    # Print Result :
+    print "\n";
+    print "---------" x 8 . "\n";
+    print " List GPG Auth : \n";
+    print "---------" x 8 . "\n";
+
+    print "Name : $name_gpg\n";
+    print "Email : $email_gpg\n";
+    print "passphrase : $passphrase_gpg\n\n";
+}
+# Subroutine for get list gpg auth with Decode passphrase:
+# ------------------------------------------------------------------------
+sub _gpg_auth_dec {
+    # Get data Setup :
+    my $data_setup = data_setup();
+    my $dir_dev = $data_setup->{'dir_dev'};
+    my $prefix_flcfg = $data_setup->{'prefix_flcfg'};
+    my $file_cfg_ext = $data_setup->{'fileCfg_ext'};
+
+    # Get All Data Config :
+    my $build = $allconfig->{'build'};
+    my $build_rilis = $build->{'rilis'};
+    my $build_gpg = $build->{'gpg'};
+    my $name_gpg = $build_gpg->{'name'};
+    my $email_gpg = $build_gpg->{'email'};
+    my $passphrase_gpg = $build_gpg->{'passphrase'};
+    my $thepassphrase = dec_gpg_genkey($email_gpg, $passphrase_gpg);
+
+    # Print Result :
+    print "\n";
+    print "---------" x 8 . "\n";
+    print " List GPG Auth : \n";
+    print "---------" x 8 . "\n";
+
+    print "Name : $name_gpg\n";
+    print "Email : $email_gpg\n";
+    print "passphrase : $thepassphrase\n\n";
+}
+# Subroutine for get list configure :
+# ------------------------------------------------------------------------
+sub _list_cfg {
+    my $data_setup = data_setup();
+    my $dir_dev = $data_setup->{'dir_dev'};
+    my $prefix_flcfg = $data_setup->{'prefix_flcfg'};
+    my $file_cfg_ext = $data_setup->{'fileCfg_ext'};
+
+    # Get Data current Configure :
+    my $prepare = $allconfig->{prepare};
+    my $build = $allconfig->{build};
+    my $build_rilis = $build->{'rilis'};
+    my $build_pgp = $build->{'gpg'};
+    my $pgp_name = $build_pgp->{'name'};
+    my $pgp_email = $build_pgp->{'email'};
+    my $bzr = $allconfig->{bzr};
+    my $bzr_url = $bzr->{'url'};
+    my $git = $allconfig->{git};
+    my $git_url = $git->{'url'};
+    my $pkg = $allconfig->{'pkg'};
+    my $dirpkg = $pkg->{'dirpkg'};
+
+    # Print Result :
+    print "\n";
+    print "---------" x 8 . "\n";
+    print " List Configure : \n";
+    print "---------" x 8 . "\n";
+    print "\n";
+
+    print "PGP Configure :\n";
+    print "Name : $pgp_name\n";
+    print "Email : $pgp_email\n\n";
+
+    print "URL Configure : \n";
+    print "Bzr Branch : $bzr_url\n";
+    print "Git Branch : $git_url\n\n";
+
+    print "Packages Configure : \n";
+    print "Package Directory : $dirpkg\n\n";
+}
+# Subroutine for list file configure :
+# ------------------------------------------------------------------------
+sub _list_file {
+    my $data_setup = data_setup();
+    my $dir_dev = $data_setup->{'dir_dev'};
+    my $prefix_flcfg = $data_setup->{'prefix_flcfg'};
+    my $file_cfg_ext = $data_setup->{'fileCfg_ext'};
+
+    opendir(my $dir, $dir_dev) or die "Cannot open directory: $!";
+    my @files = grep { $_ ne '.' && $_ ne '..' && $_ ne 'packages' } readdir $dir;
+    closedir $dir;
+
+    print "\n";
+    print "---------" x 8 . "\n";
+    print " List File Configure : \n";
+    print "---------" x 8 . "\n";
+    print "\n";
+
+    print "Directory Location : $dir_dev\n";
+
+    my $i = 0;
+    while ($i < scalar @files) {
+        if ($files[$i] =~ m/\.config$/) {
+            if ($files[$i] =~ m/tambora/) {
+                print "Tambora : $files[$i]\n";
+            }
+            if ($files[$i] =~ m/uluwatu/) {
+                print "Uluwatu : $files[$i]\n";
+            }
+        }
+        $i++;
+    }
+    print "\n";
+}
 # Subroutine for option "config" :
 # ------------------------------------------------------------------------
 sub config {
+    my $timezone;
     my $confirmation;
     my $cache_auth;
     my $gnupg_genkey;
     my $gitname;
     my $gitemail;
     my $r_gitset = 1;
+    my $read_fileCfg;
     my $home_dir = $ENV{"HOME"};
 
+
     # Get Command :
     # ----------------------------------------------------------------
     my $get_cmd = cmd_list();
@@ -135,9 +271,19 @@ sub config {
     my $gitCmd_name = $getGit_cmd->{'cfg-name'};
     my $gitCmd_email = $getGit_cmd->{'cfg-email'};
     my $gitCmd_authCache = $getGit_cmd->{'cfg-credential-cache'};
+    my $gitCmd_authCache_clear = $getGit_cmd->{'cfg-creden-cache-clear'};
     my $gitCmd_list = $getGit_cmd->{'cfg-list'};
     my $gnupg_cmd = $get_cmd->{'gpg'};
 
+    # For TimeZone :
+    # ----------------------------------------------------------------
+    print "Example name TimeZone \"Asia/Makassar\" \n";
+    print "Set your timezone name : ";
+    chomp($timezone = <STDIN>);
+    if ($timezone ne '') {
+        $time_zone = $timezone;
+    }
+
     # For GitHub Configure
     # ------------------------------------------------------------------------
     if (-e $home_dir."/.gitconfig") {
@@ -146,7 +292,7 @@ sub config {
         chomp($confirmation = <STDIN>);
         if ($confirmation eq 'y') {
 
-            my $read_fileCfg = BlankOnDev::Utils::file->read($home_dir."/.gitconfig");
+            $read_fileCfg = BlankOnDev::Utils::file->read($home_dir."/.gitconfig");
             my $name_git;
             my $email_git;
             if ($read_fileCfg =~ m/(name)\s(\=)\s(.*)/) {
@@ -167,6 +313,7 @@ sub config {
             if ($gitemail eq '') {
                 $gitemail = $email_git;
             }
+
             if ($gitname ne '' and $gitemail ne '') {
                 system("$gitCmd_name \"$gitname\"");
                 system("$gitCmd_email \"$gitemail\"");
@@ -194,11 +341,28 @@ sub config {
         }
     }
 
-    # For cache user and password git push :
-    print "You want cache user and password git [y/n]: ";
-    chomp($cache_auth = <STDIN>);
-    if ($cache_auth eq 'y' or $cache_auth eq '') {
-        system("$gitCmd_authCache --timeout=86400");
+    $read_fileCfg = BlankOnDev::Utils::file->read($home_dir."/.gitconfig");
+    my $auth_cache_git;
+    if ($read_fileCfg =~ m/(helper)\s(\=)\s(.*)/) {
+        # For cache user and password git push :
+        print "Cache user and password is activated\n";
+        print "You want to clear [y/n]: ";
+        chomp($cache_auth = <STDIN>);
+        if ($cache_auth eq 'y') {
+#            $read_fileCfg =~ s/^\[credential.*//g;
+#            $read_fileCfg =~ s/(^\s+helper.*)+//g;
+#            chmod 0666, $home_dir.'/.gitcontif';
+#            BlankOnDev::Utils::file->create('/.gitconfig', $home_dir, $read_fileCfg);
+#            chmod 0644, $home_dir.'/.gitcontif';
+            system($gitCmd_authCache_clear);
+        }
+    } else {
+        # For cache user and password git push :
+        print "You want cache user and password git [y/n]: ";
+        chomp($cache_auth = <STDIN>);
+        if ($cache_auth eq 'y' or $cache_auth eq '') {
+            system("$gitCmd_authCache --timeout=86400");
+        }
     }
 
     # get List git config :
@@ -216,6 +380,7 @@ sub config {
 # ------------------------------------------------------------------------
 sub data_setup {
     my $data_dev = BlankOnDev::DataDev::data_dev();
+
     return $data_dev;
 }
 # Subroutine for Set name BlankOn Rilis :
@@ -234,6 +399,8 @@ sub boi_rilis {
 # ------------------------------------------------------------------------
 sub enc_ggp_genkey {
     my ($email, $passphrase) = @_;
+#    print "Input Email : $email\n";
+#    print "Input Passphrase : $passphrase\n";
 
     my $plan_key = BlankOnDev::enkripsi->getKey_enc($email);
     my $encoder = BlankOnDev::enkripsi->Encoder($passphrase, $plan_key);
@@ -244,9 +411,14 @@ sub enc_ggp_genkey {
 # ------------------------------------------------------------------------
 sub dec_gpg_genkey {
     my ($email, $passphrase) = @_;
+#    print "Input Email : $email\n";
+#    print "Input Passphrase : $passphrase\n";
 
     my $plan_key = BlankOnDev::enkripsi->getKey_enc($email);
     my $decoder = BlankOnDev::enkripsi->Decoder($passphrase, $plan_key);
+    $decoder =~ s/\|+//g;
+
+    return $decoder;
 }
 # Subroutine for GNUpg configure :
 # ------------------------------------------------------------------------
@@ -277,7 +449,7 @@ sub gpg_config {
     my $curren_cfg = $allconfig;
     my $build_cfg = $allconfig->{'build'};
     my $build_gpg = $build_cfg->{'gpg'};
-    if (exists $build_cfg->{'name'}) {
+    if (exists $build_gpg->{'name'}) {
         $curr_gpg_name = '['.$build_gpg->{'name'}.']' if $build_gpg->{'name'} ne '';
         $curr_gpg_email = '['.$build_gpg->{'email'}.']' if $build_gpg->{'email'} ne '';
         $curr_gpg_passph = '['.$build_gpg->{'passphrase'}.']' if $build_gpg->{'passphrase'} ne '';
@@ -312,21 +484,21 @@ sub gpg_config {
     }
 
     # Form Name GnuPG generate key :
-    print "Enter name for GnuPG generate key $curr_gpg_name : ";
+    print "Enter Name $curr_gpg_name : ";
     chomp($gpg_name = <STDIN>);
     if ($gpg_name eq '') {
         $gpg_name = $currdt_gpg_name;
     }
 
     # Form Email for GnuPG generate key :
-    print "Enter E-mail for GnuPG generate key $curr_gpg_email : ";
+    print "Enter E-mail $curr_gpg_email : ";
     chomp($gpg_email = <STDIN>);
     if ($gpg_email eq '') {
         $gpg_email = $currdt_gpg_email;
     }
 
     # From PassPhrase for GnuPG generate key :
-    print "Enter passphrase for GnuPG generate key :";
+    print "Enter passphrase : ";
     ReadMode('noecho');
     $gpg_passph = ReadLine(0);
     if ($gpg_passph eq '') {
@@ -381,7 +553,7 @@ sub bzr_config {
 
     # Form :
     my $data_bzrcfg = $allconfig->{'bzr'};
-    print "URL BZR : $data_bzrcfg\n";
+#    print "URL BZR : $data_bzrcfg\n";
     my $url_bzr = $data_bzrcfg->{'url'} if exists $data_bzrcfg->{'url'};
     if ($url_bzr eq '') {
         print "Enter bzr url : ";
@@ -393,7 +565,7 @@ sub bzr_config {
 
     if ($bzr_url eq '') {
         $bzr_url = {
-            'url' => ''
+            'url' => $allconfig->{'bzr'}->{'url'}
         };
         $r_bzrcfg = 0;
     } else {
@@ -427,6 +599,7 @@ sub git_config {
         };
         $r_gitcfg = 0;
     } else {
+        $git_url =~ s/\/$//g;
         $gitCfg = {
             'url' => $git_url,
         };
@@ -437,12 +610,14 @@ sub git_config {
 # ------------------------------------------------------------------------
 sub read_config {
     my $data = '';
+    my %data_pkg = ();
     my $data_setup = data_setup();
     my $dir_dev = $data_setup->{'dir_dev'};
     my $prefix_file_cfg = $data_setup->{'prefix_flcfg'};
     my $ext_flcfg = $data_setup->{'fileCfg_ext'};
     my $home_dir = $ENV{"HOME"};
     my $pkgs_dir = $data_setup->{'dir_pkg'};
+    my $logs_dir = $data_setup->{'dirlogs'};
 
     # For Release set :
     # ----------------------------------------------------------------
@@ -452,8 +627,11 @@ sub read_config {
     my $locdir_pkg = $dir_dev.$pkgs_dir;
     $filename_cfg = $file_cfg;
     $dirdev_cfg = $dir_dev;
+    my $adddt_pkg;
+    my $result_adddtPkg;
 
     # Define scalar :
+    my $timezone;
     my $prepare;
     my $build;
     my $build_rilis;
@@ -466,6 +644,7 @@ sub read_config {
     my $dir_data_boidev = $dir_dev;
     my $dir_pkg = $dir_dev.$pkgs_dir;
     my $dir_pkgrilis = $dir_pkg.'/'.$rilisCfg;
+    my $log_dir_rilis = $logs_dir.$rilisCfg;
 
     # Get format data config :
     my $format_config = format_data();
@@ -477,6 +656,7 @@ sub read_config {
             my $data_allcfg = decode_json($get_allcfg);
             my $size_allcfg = scalar keys(%{$data_allcfg});
             if ($size_allcfg > 0) {
+                $timezone = $data_allcfg->{'timezone'} if exists $data_allcfg->{'timezone'};
                 $prepare = $data_allcfg->{'prepare'} if exists $data_allcfg->{'prepare'};
                 $build = $data_allcfg->{'build'} if exists $data_allcfg->{'build'};
                 $build_rilis = $build->{'rilis'} if exists $build->{'rilis'};
@@ -494,17 +674,34 @@ sub read_config {
 #                $data_format->{'pkg'} = $pkg;
                 $data = $data_allcfg;
             } else {
-                $format_config->{'pkg'}->{'dirpkg'} = $dir_pkgrilis;
+                $adddt_pkg = Hash::MultiValue->new();
+                $adddt_pkg->add('dirpkg' => $dir_pkgrilis);
+                $adddt_pkg->add('group' => {});
+                $adddt_pkg->add('pkgs' => {});
+                $result_adddtPkg = $adddt_pkg->as_hashref;
+                $format_config->{'pkg'} = $result_adddtPkg;
                 $data = $format_config;
             }
         } else {
             BlankOnDev::Utils::file->create($file_cfg, $dir_dev, encode_json($format_config));
-            $format_config->{'pkg'}->{'dirpkg'} = $dir_pkgrilis;
+#            $format_config->{'pkg'}->{'dirpkg'} = $dir_pkgrilis;
+            $adddt_pkg = Hash::MultiValue->new();
+            $adddt_pkg->add('dirpkg' => $dir_pkgrilis);
+            $adddt_pkg->add('group' => {});
+            $adddt_pkg->add('pkgs' => {});
+            $result_adddtPkg = $adddt_pkg->as_hashref;
+            $format_config->{'pkg'} = $result_adddtPkg;
             $data = $format_config;
         }
     } else {
         mkdir($dir_data_boidev);
-        $format_config->{'pkg'}->{'dirpkg'} = $dir_pkgrilis;
+#        $format_config->{'pkg'}->{'dirpkg'} = $dir_pkgrilis;
+        $adddt_pkg = Hash::MultiValue->new();
+        $adddt_pkg->add('dirpkg' => $dir_pkgrilis);
+        $adddt_pkg->add('group' => {});
+        $adddt_pkg->add('pkgs' => {});
+        $result_adddtPkg = $adddt_pkg->as_hashref;
+        $format_config->{'pkg'} = $result_adddtPkg;
         $data = $format_config;
     }
 
@@ -524,6 +721,22 @@ sub read_config {
         }
     }
 
+    # For Logs :
+    unless (-d $logs_dir) {
+        mkdir($logs_dir);
+        unless (-d $log_dir_rilis) {
+            mkdir($log_dir_rilis);
+        }
+    } else {
+        unless (-d $log_dir_rilis) {
+            mkdir($log_dir_rilis);
+        }
+    }
+
+#    BlankOnDev::Migration::bazaar2GitHub::tmp_cfg->first_addpkg_fileTmp($data_setup, $rilisCfg);
+
+#    print Dumper $data;
+
     $allconfig = $data;
     $r_config = $data;
     return $data;
@@ -532,6 +745,7 @@ sub read_config {
 # ------------------------------------------------------------------------
 sub format_data {
     my %data = (
+        'timezone' => 'Asia/Makassar',
         'prepare' => 0,
         'build' => {
             'rilis' => '',
@@ -564,6 +778,7 @@ sub cmd_list {
         'cfg-name' => 'git config --global user.name',
         'cfg-email' => 'git config --global user.email',
         'cfg-credential-cache' => 'git config --global credential.helper cache',
+        'cfg-creden-cache-clear' => 'git config --global --unset credential.helper',
         'cfg-list' => 'git config --list',
     };
     # GPG Command :
