@@ -109,10 +109,8 @@ sub Encoder {
     # Final Random :
     my $data = $self->random($temp2, $num11, $num21, 2);
 
-    print
-
-        # Return :
-        return $data;
+    # Return :
+    return $data;
 }
 # End of Subroutine for Web Encoder
 # ===========================================================================================================
@@ -228,7 +226,7 @@ sub Decoder {
     #	print "--- Batas Step 3 ---\n";
 
     # Extract Loop :
-    $data = $self->extract_random($temp2, $num1, $num2, 2);
+    $data = $self->extract_random($temp2, $num11, $num21, 2);
     #	$data = $temp2;
 
     # Return result :
